/////////////////////////////////////////////////////////////////////////////
// This software is distributed under the terms of the General Public License.
//
// Program : BuildIT!
// Authors : Artur Prusakiewicz & Michal Switala
// E-mail  : prusak@mat.uni.torun.pl
//           scisly@mat.uni.torun.pl
// WWW     : http://www.mat.uni.torun.pl/~scisly/buildit/
//           http://www.mat.uni.torun.pl/~prusak/buildit/
//////////////////////////////////////////////////////////////////////////////


#include "Generator.h" 


Generator::Generator(QProgressBar *p,QWidget *widget, QString home, QString sys, int st)
{
  counter=0;
  homeDir=home.copy();
  sysDir=sys.copy();
  progress=p;
  
  first = TRUE;
  tmpcounter = 0;
  l1 = 0;
  l2 = 0;
  l3 = 0;
  l4 = 0;
  numgroups = 0;
  numcanvas = 0;
  
  style = st;
  
  for (int i = 0; i < 11; i++)
    listener[i] = FALSE;
  clearTab();
  
  listenertab[0] = "ActionListener";
  listenertab[1] = "AdjustmentListener";
  listenertab[2] = "ComponentListener";
  listenertab[3] = "ContainerListener";
  listenertab[4] = "FocusListener";
  listenertab[5] = "ItemListener";
  listenertab[6] = "KeyListener";
  listenertab[7] = "MouseListener";
  listenertab[8] = "MouseMotionListener";
  listenertab[9] = "TextListener";
  listenertab[10] = "WindowListener";
  
  scope[0] = "private";
  scope[1] = "protected";
  scope[2] = "public";
}

void Generator::clearTab()
{
  for (int i = 0; i < 25; i++)
    evntTab[i] = FALSE;
}

QString Generator::replaceMarks(QString *b, int n, QString rep[])
{
  QString *ret = new QString(200);
  int pos, i, count;

  *ret = b->copy();
  count = 0;
  for (i = 0; i < n ; i++) 
    {
      QString s;
      s.sprintf("<mark%d>", i);
      pos = ret->find((const char*) s);
      while (pos != -1) 
	{
	  if (pos != -1) 
	    {
	      if (i < 10)
		ret->replace(pos, 7, (const char*) rep[i]);
	       else 
		 if (i < 100) ret->replace(pos, 8, (const char*) rep[i]);
		  else 
		    ret->replace(pos, 9, (const char*) rep[i]);
	    }
	  pos = ret->find((const char*) s, pos+1);
	}
    } 

  return *ret;
}
    
void Generator::concateFiles(QString name1, QString name2, bool append)
{
  QFile f1(name1);
  QFile f2(name2);
  
  if (append) 
    {
      if (f1.open(IO_WriteOnly | IO_Append) == FALSE)
	{
	  printf("Generator::concateFiles => cannot open file=%s to write\n", name1.data());
	  exit(1);
	}
    }
  else 
    {
      if (f1.open(IO_WriteOnly) == FALSE) 
	{
	  printf("Generator::concateFiles => cannot open file=%s to write\n", name1.data());
	  exit(1);
	}
    }
  if (f2.open(IO_ReadOnly) == FALSE)
    {
      printf("Generator::concateFiles => cannot open file=%s to read\n", name2.data());
      exit(1);
    }

  QTextStream t( &f2 );
  QString s;          
 
  while ( !t.eof() ) 
    {
      s = t.readLine(); 
      s.append("\n");
      f1.writeBlock(s, s.length());
    }             
  f1.close();
  f2.close();
}

void Generator::removeComments(QString filename)
{
  QFile filein(filename);
  QFile fileout(getTmpName());
  
  if (filein.open(IO_ReadOnly) == FALSE) 
	{
	  printf("Generator::removeComments => cannot open file=%s to read\n", filename.data());
	  exit(1);
	}

  if (fileout.open(IO_WriteOnly) == FALSE) 
	{
	  printf("Generator::removeComments => cannot open file=%s to write\n", (getTmpName()).data());
	  exit(1);
	}

  QTextStream t( &filein );
  QString s;          
 
  while ( !t.eof() ) 
    {
      s = t.readLine(); 
      s.append("\n");
      if (s.contains("//") == 0)
	  fileout.writeBlock(s, s.length());
      else 
	  fileout.writeBlock(QString("\n"), 1);
    }           
  filein.close();
  fileout.close();
  copyFile(filename, getTmpName());
}

void Generator::removeSpace(QString dfilename, QString sfilename)
{
  QFile f1(dfilename);
  QFile f2(sfilename);
  
  if (f1.open(IO_WriteOnly) == FALSE) 
    {
      printf("Generator::removeSpace => cannot open file=%s to write\n", dfilename.data());
      exit(1);
    }

  if (f2.open(IO_ReadOnly) == FALSE)
    {
      printf("Generator::removeSpace => cannot open file=%s to read\n", sfilename.data());
      exit(1);
    }

  QTextStream t( &f2 );
  QString s;   
  bool writeallow;
  while ( !t.eof() ) 
    {
      writeallow = TRUE;
      s = t.readLine(); 
      if (s.isEmpty())
	{
	  s = t.readLine();
	  if (s.isEmpty())
	    {
	      s = t.readLine();
	      if (s.isEmpty())
		{
		  writeallow = FALSE;
		  s = '\0';
		}
	      else s.insert(0,"\n");
	    }
	  else
	    s.insert(0,"\n");
	}
      s.append("\n");
      if (writeallow) 
	f1.writeBlock(s, s.length());
    }             
  f1.close();
  f2.close();
}

void Generator::copyFile(QString dfilename, QString sfilename)
{
  concateFiles(dfilename.copy(), sfilename.copy(), FALSE);
}

void Generator::insertBuffer(QString dfilename, QString sfilename, QString *buf, int line)
{
  QFile f1(dfilename);
  QFile f2(sfilename);
  if (f1.open(IO_WriteOnly) == FALSE)
    {
      printf("Generator::insertBuffer => cannot open file=%s to write\n", dfilename.data());
      exit(1);
    }
  if (f2.open(IO_ReadOnly) == FALSE)
    {
      printf("Generator::insertBuffer => cannot open file=%s to read %s\n", sfilename.data(), dfilename.data());
      exit(1);
    }

  QTextStream t1( &f2 );
  QString s;         
  int l;

  l = 1;
  while ( l < line ) 
    {
      s = t1.readLine(); 
      s.append("\n");
      f1.writeBlock(s, s.length());
      l++;
    }             
  f1.writeBlock((const char*) *buf, buf->length());
  while ( !t1.eof() ) 
    {
      s = t1.readLine();
      s.append("\n");
      f1.writeBlock(s, s.length());
    }
  f1.close();
  f2.close();
}

void Generator::insertFile(QString dest, QString source1, QString source2, int line)
{
  QFile f1(dest);
  QFile f2(source1);
  QFile f3(source2);

  if (f1.open(IO_WriteOnly) == FALSE)
    {
      printf("Generator::insertFile => cannot open file=%s to write\n", dest.data());
      exit(0);
    }
  if (f2.open(IO_ReadOnly) == FALSE)
    {
      printf("Generator::insertFile => cannot open file=%s to read 1\n", source1.data());
      exit(1);
    }
  if (f3.open(IO_ReadOnly) == FALSE)
    {
      printf("Generator::insertFile => cannot open file=%s to read 2\n", source2.data());
      exit(1);
    }      

  QTextStream t1( &f2 );
  QString s;         
  int l;

  l = 1;
  while ( l < line ) 
    {
      s = t1.readLine(); 
      s.append("\n");
      f1.writeBlock(s, s.length());
      l++;
    }             
  
  QTextStream t2( &f3 );
  while ( !t2.eof() ) 
    {
      s = t2.readLine();
      s.append("\n");
      f1.writeBlock(s, s.length());
    }

  while ( !t1.eof() ) 
    {
      s = t1.readLine();
      s.append("\n");
      f1.writeBlock(s, s.length());
    }
  f1.close();
  f2.close();
  f3.close();
}

void Generator::template2Source(QString dfilename, QString sfilename, int n, QString arg[], bool ap)
{
  QFile in(sfilename);
  QFile out(dfilename);

  if (in.open(IO_ReadOnly) == FALSE) 
    {
      printf("Generator::template2Source => cannot open file=%s to read\n", sfilename.data());
      exit(1);
    }
  if (!ap)
    if (out.open(IO_WriteOnly) == FALSE) 
      {
	printf("Generator::tempalte2Source => cannot open file=%s to write\n", dfilename.data());
	exit(1);
      }
  QString s;
  QTextStream t( &in );

  while ( !t.eof() ) 
    {
      s = t.readLine();
      s.append("\n");
      s = replaceMarks(&s, n, arg);
      out.writeBlock(s, s.length());
    }
  in.close();
  out.close();
}

int Generator::getFileLines(QString filename)
{
  QFile f(filename);
  if (f.open(IO_ReadOnly) == FALSE) 
    {
      printf("Generator::getFileLines => cannot open file=%s to read\n", filename.data());
      exit(1);
    }
    
  QString s;
  QTextStream t(&f);
  int result = 0;
  int empty = 0;
  while ( !t.eof() )
    {
      s = t.readLine();
      if (!s.isEmpty()) 
	empty = 1;
      else 
	empty++;
      result++;
    }
  f.close();
  return (result - empty);
}


QString Generator::getTmpName()
{
  QString s = homeDir + ".build/tmp/buildit.tmp";
  return s;
}

void Generator::addNewCheckBoxGroup(QString name)
{
  if (numgroups < 9)
    boxgroups[numgroups++] = name.copy();
  else printf("Generator::addNewCheckBoxGroup ==> bad number of group\n");
}

bool Generator::isCheckBoxGroup(QString name)
{
  int i;

  for (i = 0; i < numgroups; i++) 
    if (boxgroups[i] == name) return TRUE;
  
  return FALSE;
}

QString Generator::getHigherMenu(MyTree *t, char* name, int num)
{
  int i;
  QString t1(name);
  QString t2, t3;
  
  t2 = t1.stripWhiteSpace();
  int sub = t1.length() - t2.length();
  if (!sub)
    {
      QString tname = t->GetFeature1(1)->copy();
      tname = tname.right(tname.length()-tname.findRev(" ")-1);
      QString rname = tname.copy();
      if (t->GetFeature2(24) == 1)
        rname = rname + "_";
      return rname.copy();
    }
  for (i = num-1; i >=0; i--)
    {
      int sub2;
      t2 = QString(t->GetChildren(i)->GetFeature1(3)->data());
      t3 = t2.stripWhiteSpace();
      sub2 = t2.length() - t3.length();
      if ((sub - sub2) == 1)
        {
          if (t->GetChildren(i)->GetFeature2(24) == 1)
            t3 = t3 + "_";
	  return t3.copy();
        }
    }
  
  return QString("notfound");
}
  
void Generator::updateListener(QString n, bool tab[])
{
  int z = -1;
  QString name = n.copy();
  if (name.contains("action") > 0) 
    {
      tab[0] = TRUE;
      listener[0] = TRUE;
      z = 0;
      
    }
  if (name.contains("adjustment") > 0) 
    {
      tab[1] = TRUE;
      listener[1] = TRUE;
      z = 1;
    }
  if (name.contains("component") > 0) 
    {
      if ((name.contains("Added") == 0) && (name.contains("Removed") == 0))
	{
	  tab[2] = TRUE;
	  listener[2] = TRUE;
	  z = 2;
	  if (name.contains("Resized") > 0) 
	    evntTab[0] = TRUE;
	  if (name.contains("Moved") > 0) 
	    evntTab[1] = TRUE;
	  if (name.contains("Shown") > 0) 
	    evntTab[2] = TRUE;
	  if (name.contains("Hidden") > 0) 
	    evntTab[3] = TRUE;
	}
      else
	{
	  tab[3] = TRUE;
	  listener[3] = TRUE;
	  z = 3;
	  if (name.contains("Added") > 0)
	    evntTab[4] = TRUE;
	  else 
	    evntTab[5] = TRUE;
	}
    }
  if (name.contains("focus") > 0) 
    {
      tab[4] = TRUE;
      listener[4] = TRUE;
      z = 4;
      if (name.contains("Gained") > 0)
	evntTab[6] = TRUE;
      else 
	evntTab[7] = TRUE;
    } 
  if (name.contains("item") > 0) 
    {
      tab[5] = TRUE;
      listener[5] = TRUE;
      z = 5;
    } 
  if (name.contains("key") > 0) 
    {
      tab[6] = TRUE;
      listener[6] = TRUE;
      z = 6;
      if (name.contains("Pressed") > 0)
	evntTab[8] = TRUE;
      if (name.contains("Typed") > 0)
	evntTab[9] = TRUE;
      if (name.contains("Released") > 0)
	evntTab[10] = TRUE;
    } 
  if (name.contains("mouse") > 0) 
    {
      if ((name.contains("Dragged") == 0) && (name.contains("Moved") == 0))
	{
	  tab[7] = TRUE;
	  listener[7] = TRUE;
	  z = 7;
	  if (name.contains("Clicked") > 0)
	    evntTab[11] = TRUE;
	  if (name.contains("Pressed") > 0)
	    evntTab[12] = TRUE;
	  if (name.contains("Released") > 0)
	    evntTab[13] = TRUE;
	  if (name.contains("Entered") > 0)
	    evntTab[14] = TRUE;
	  if (name.contains("Exited") > 0)
	    evntTab[15] = TRUE;
	}
      else
	{
	  tab[8] = TRUE;
	  listener[8] = TRUE;
	  z = 8;
	  if (name.contains("Dragged") > 0)
	    evntTab[16] = TRUE;
	  else
	    evntTab[17] = TRUE;
	}
    }
  if (name.contains("text") > 0) 
    {
      tab[9] = TRUE;
      listener[9] = TRUE;
      z = 9;
    } 
  if (name.contains("window") > 0) 
    {
      tab[10] = TRUE;
      listener[10] = TRUE;
      z = 10;
      if (name.contains("Activated") > 0)
	evntTab[18] = TRUE;
      if (name.contains("Deactivated") > 0)
	evntTab[19] = TRUE;
      if (name.contains("Deiconified") > 0)
	evntTab[20] = TRUE;
      if (name.contains("Iconified") > 0)
	evntTab[21] = TRUE;
      if (name.contains("Closed") > 0)
	evntTab[22] = TRUE;
      if (name.contains("Closing") > 0)
	evntTab[23] = TRUE;
      if (name.contains("Opened") > 0)
	evntTab[24] = TRUE;
    } 
}

void Generator::insertOneEvent(QString event, QString *tab)
{
  template2Source(homeDir+".build/tmp/event.tmp", sysDir+"templates/events/style2/"+event.copy(), 1, tab, FALSE);
  insertFile(getTmpName(), getTargetName() , homeDir+".build/tmp/event.tmp", l3);
  l3 += 7;
  copyFile(getTargetName(), getTmpName());
}

void Generator::insertAdapter(QString clname, QString adname, int from, int to)
{
  QString buf;
  int i, counter = 1;
  buf="    class "+clname.copy()+" extends "+adname.copy()+"Adapter {\n\n\n    }\n";
  insertBuffer(getTmpName(), getTargetName(), &buf, l3);
  copyFile(getTargetName(), getTmpName());
  l3++;
  for (i = from; i <= to; i++)
    {
      if (evntTab[i])
	{
	  insertFile(getTmpName(), getTargetName(), sysDir+"templates/events/style2/"+adname.lower()+QString().setNum(counter), l3);
	  l3 += 4;
	  copyFile(getTargetName(), getTmpName());
	}
      counter++;
    }
  l3 += 3;
}

void Generator::addEvent(QString filename, QString varname, int num)
{
  QString classname;
  QString mark[7];

  classname = varname.copy()+"L"+QString().setNum(num);
  mark[0] = classname.copy();
  switch (num) 
    {
    case ACL: 
      insertOneEvent("action", mark);
      break;
    case ADL:
      insertOneEvent("adjustment", mark);
      break;
    case IL:
      insertOneEvent("item", mark);
      break;
    case TL:
      insertOneEvent("text", mark);
      break;
    case CML:
      insertAdapter(classname, "Component", 0, 3);
      break;
    case CNL:
      insertAdapter(classname, "Container", 4, 5);
      break;
    case FL:
      insertAdapter(classname, "Focus", 6, 7);
      break;
    case KL:
      insertAdapter(classname, "Key", 8, 10);
      break;
    case ML:
      insertAdapter(classname, "Mouse", 11, 15);
      break;
    case MML:
      insertAdapter(classname, "MouseMotion", 16, 17);
      break;
    case WL:
      insertAdapter(classname, "Window", 18, 24);
      break;
    }
}

void Generator::addListener(MyTree *t, QString f1name, QString f2name, QString varname, int &line)
{
  QString lbuf;
  bool state[11];
  int i;

  if (style) clearTab();
  for (i=0; i<11; i++) state[i] = FALSE;
  int num = t->GetFeature2(19);
  for (i=0; i<num; i++) 
    updateListener(t->GetFeature1(10+i)->copy(), state);
  for (i=0; i<11; i++)
    if (state[i])
      {
	QString classname = varname.copy() + "L" + QString().setNum(i);
	if (*t->GetFeature1(2) == "MyClass")
	  {
	    if (!style)
	      lbuf = "        add" + listenertab[i].copy() + "( this );\n";
	    else
	      lbuf = "        add" + listenertab[i].copy() + "( new " + classname.copy() + "() );\n          //  To write code for event(s) from above Listener\n          //  look for class \"" +classname.copy() +"\"\n        /////////////////////////////////////////\n";
	  }
	else
	  {
	    if (!style)
	      lbuf = "        " + varname.copy() + ".add" + listenertab[i].copy() + "( this );\n";
	    else
	      lbuf="        "+varname.copy()+".add"+listenertab[i].copy()+"( new "+classname.copy()+"() );\n          //  To write code for "+varname.copy()+"'s event(s) from above Listener\n          //  look for class \""+classname.copy()+"\"\n        ///////////////////////////////////\n";
	  }
	insertBuffer(f1name.copy(), f2name.copy(), &lbuf, line);
	copyFile(f2name, f1name);
	
	if (!style) 
	  {
	    line++;
	    l3++;
	  }
	else
	  {
	    line += 4;
	    l3 += 4;
	    addEvent(f2name, varname, i);
	  }
      }
}

void Generator::importHelp(QString filename, QString name)
{
  QString tempstr = sysDir + "templates/events/" + name.lower();

  copyFile(filename, getTmpName());
  int line = getFileLines(filename);
  insertFile(getTmpName(), filename, tempstr, line);
  copyFile(filename, getTmpName());
}

void Generator::addImport(QString filename)
{
  int i;
  QString buf;
  
  
  for (i=0; i<11; i++)
    {
      if (listener[i])
	{
	  switch (i) 
	    {
	    case 0:
	      importHelp(filename, "Action");
	      break;
	    case 1:
	      importHelp(filename, "Adjustment");
	      break;
	    case 2:
	      importHelp(filename, "Component");
	      break;
	    case 3:
	      importHelp(filename, "Container");
	      break;
	    case 4:
	      importHelp(filename, "Focus");
	      break;
	    case 5:
	      importHelp(filename, "Item");
	      break;
	    case 6:
	      importHelp(filename, "Key");
	      break;
	    case 7:
	      importHelp(filename, "Mouse");
	      break;
	    case 8:
	      importHelp(filename, "MouseMotion");
	      break;
	    case 9:
	      importHelp(filename, "Text");
	      break;
	    case 10:
	      importHelp(filename, "Window");
	      break;
	    }
	}
    }
  copyFile(filename, getTmpName());
}

void Generator::implementsHelp(QString filename, QString name)
{
  

  if (first)
    {
      QString buf = "  implements " + name.copy() + "Listener\n";
      insertBuffer(getTmpName(), filename, &buf, l4);
      l4++;
      first = FALSE;
    }
  else
    {
      QString buf = "  ," + name.copy() + "Listener\n";
      insertBuffer(getTmpName(), filename, &buf, l4);
      l4++;
    }
  copyFile(filename, getTmpName());
}

void Generator::addImplements(QString filename)
{
  int i, line;
  QString buf;
  
  for (i=0; i<11; i++)
    {
      if (listener[i])
	{
	  switch (i) 
	    {
	      case 0:
	      implementsHelp(filename, "Action");
	      break;
	    case 1:
	      implementsHelp(filename, "Adjustment");
	      break;
	    case 2:
	      implementsHelp(filename, "Component");
	      break;
	    case 3:
	      implementsHelp(filename, "Container");
	      break;
	    case 4:
	      implementsHelp(filename, "Focus");
	      break;
	    case 5:
	      implementsHelp(filename, "Item");
	      break;
	    case 6:
	      implementsHelp(filename, "Key");
	      break;
	    case 7:
	      implementsHelp(filename, "Mouse");
	      break;
	    case 8:
	      implementsHelp(filename, "MouseMotion");
	      break;
	    case 9:
	      implementsHelp(filename, "Text");
	      break;
	    case 10:
	      implementsHelp(filename, "Window");
	      break;
	    }
	}
    }
}

void Generator::genApplet(MyTree *t)
{
  QString tname;
  QString mark[6];
  QString name;
  
  l1 = 10;
  l2 = 18;
  l3 = 36;
  l4 = 6;
  tname = t->GetFeature1(1)->copy();
  tname = tname.right(tname.length()-tname.findRev(" ")-1);
  name = tname.copy();
  name.right(name.length() - name.findRev("/"));
  mark[0] = name.copy();
  mark[1] = mark[1].setNum(t->GetFeature2(3));
  mark[2] = mark[2].setNum(t->GetFeature2(4)+menuy);
  mark[3] = mark[3].setNum(t->GetFeature2(5));
  mark[4] = mark[4].setNum(t->GetFeature2(6));
  mark[5] = mark[5].setNum(t->GetFeature2(7));
  template2Source(homeDir + ".build/tmp/applet.tmp",sysDir+"templates/applet.jbt", 6, mark, FALSE);
 
  if (t->GetFeature2(19) > 0)
      addListener(t, homeDir+".build/tmp/listener.tmp",homeDir+ ".build/tmp/applet.tmp", name.copy(), l2);
}

void Generator::genFrame(MyTree *t)
{
  QString tname;
  QString mark[7];
  QString name;
  
  l1 = 9; // 11
  l2 = 21; // 19
  l3 = 27;
  l4 = 5;
  tname = t->GetFeature1(1)->copy();
  tname = tname.right(tname.length()-tname.findRev(" ")-1);
  name = tname.copy();
  name.right(name.length() - name.findRev("/"));
  mark[0] = name.copy();
  mark[1] = mark[1].setNum(t->GetFeature2(3));
  mark[2] = mark[2].setNum(t->GetFeature2(4)+menuy);
  mark[3] = mark[3].setNum(t->GetFeature2(5));
  mark[4] = mark[4].setNum(t->GetFeature2(6));
  mark[5] = mark[5].setNum(t->GetFeature2(7));
  mark[6] = t->GetFeature1(5)->copy();
  template2Source(homeDir+".build/tmp/frame.tmp", sysDir+"templates/frame.jbt", 7, mark, FALSE);
  if (t->GetFeature2(19) > 0)
      addListener(t, homeDir+".build/tmp/listener.tmp", homeDir+".build/tmp/frame.tmp", name.copy(), l2);
}

void Generator::genWindow(MyTree *t)
{
  QString tname;
  QString mark[6];
  QString name;
  
  l1 = 11;
  l2 = 19;
  l3 = 27;
  l4 = 5;
  tname = t->GetFeature1(1)->copy();
  tname = tname.right(tname.length()-tname.findRev(" ")-1);
  name = tname.copy();
  name.right(name.length() - name.findRev("/"));
  mark[0] = name.copy();
  mark[1] = mark[1].setNum(t->GetFeature2(3));
  mark[2] = mark[2].setNum(t->GetFeature2(4)+menuy);
  mark[3] = mark[3].setNum(t->GetFeature2(5));
  mark[4] = mark[4].setNum(t->GetFeature2(6));
  mark[5] = mark[5].setNum(t->GetFeature2(7));
  template2Source(homeDir+".build/tmp/window.tmp", sysDir+"templates/window.jbt", 6, mark, FALSE);
  if (t->GetFeature2(19) > 0)
      addListener(t, homeDir+".build/tmp/listener.tmp", homeDir+".build/tmp/window.tmp", name.copy(), l2);
}

void Generator::genCheckbox(MyTree *t)
{
  QString mark[16];

  QString temp1=t->GetFeature1(1)->copy();
  temp1 = temp1.right(temp1.length()-temp1.findRev(" ")-1);
  mark[0] = temp1.copy();
  mark[1] = t->GetFeature1(3)->copy();
  int x,y;
  x = 0;
  y = 0;
  if (*t->GetParent()->GetFeature1(2) == QString("MyCanvas"))
    {
      t->countFeature(t, 1, x);
      t->countFeature(t, 2, y);
    }
  else
    {
       x = t->GetFeature2(1);
       y = t->GetFeature2(2);
    }
  mark[2] = mark[2].setNum(x);
  mark[3] = mark[3].setNum(y+menuy);
  mark[4] = mark[4].setNum(t->GetFeature2(3));
  mark[5] = mark[5].setNum(t->GetFeature2(4));
  mark[6] = t->GetFeature1(4)->copy();
  mark[7] = "Font.PLAIN";
  if (t->GetFeature2(12) > 50) 
    mark[7].append("+Font.BOLD");
  if (t->GetFeature2(13) == 1) 
    mark[7].append("+Font.ITALIC");
  mark[8] = mark[8].setNum(t->GetFeature2(11));
  mark[9] = mark[9].setNum(t->GetFeature2(5));
  mark[10] = mark[10].setNum(t->GetFeature2(6));
  mark[11] = mark[11].setNum(t->GetFeature2(7));
  mark[12] = mark[12].setNum(t->GetFeature2(8));
  mark[13] = mark[13].setNum(t->GetFeature2(9));
  mark[14] = mark[14].setNum(t->GetFeature2(10));
  if (t->GetFeature2(20) == 0) 
    mark[15] = "false";
  else 
    mark[15] = "true";
  template2Source(homeDir+".build/tmp/checkbox.tmp", sysDir+"templates/checkbox.jbt", 16, mark, FALSE);
  
  QString buf;
  int sc = t->GetFeature2(18);
  if (sc == -1) sc = 0;
  buf = "        " + scope[sc] + " Checkbox " + temp1 + ";\n";
  bool added = FALSE;
  QString group = t->GetFeature1(5)->copy();
  group = group.stripWhiteSpace();
  if (group.isEmpty() == FALSE)
    {
      if (isCheckBoxGroup(group) == FALSE)
	  {
	    addNewCheckBoxGroup(group);
	    added = TRUE;
	    buf = buf + "        CheckboxGroup " + group.copy() + ";\n";
	  }
    }
  insertBuffer(getTmpName(), getTargetName(), &buf, l1);
  l1++;
  l2++;
  l3++;
  if (added) 
    {
      l1++;
      l2++;
      l3++;
    }
  copyFile(getTargetName(), getTmpName());
  insertFile(getTmpName(), getTargetName(), homeDir+".build/tmp/checkbox.tmp", l2);
  l2 += 8;
  l3 += 8;
  if (group.isEmpty() == FALSE)
    {
      QString buf2;
      buf2 = "";
      if (added)
	buf2 = "        " + group.copy() + " = new CheckboxGroup();\n";
      buf2 = buf2 + "        " + temp1.copy() + ".setCheckboxGroup(" + group.copy() + ");\n";
      insertBuffer(homeDir+".build/tmp/buildit2.tmp", getTmpName(), &buf2, l2);
      l2 ++;
      l3++;
      if (added) 
	{
	  l2++;
	  l3++;
	}
      copyFile(getTargetName(), homeDir+".build/tmp/buildit2.tmp");
    }
  else 
    copyFile(getTargetName(), getTmpName());
  if (t->GetFeature2(19) > 0)
      addListener(t, getTmpName(), getTargetName(), mark[0].copy(), l2);
}

void Generator::genChoice(MyTree *t)
{
  QString mark[15];
  
  QString temp1=t->GetFeature1(1)->copy();
  temp1 = temp1.right(temp1.length()-temp1.findRev(" ")-1);
  mark[0] = temp1.copy();
  mark[1] = "";
  int x,y;
  x = 0;
  y = 0;
  if (*t->GetParent()->GetFeature1(2) == QString("MyCanvas"))
    {
      t->countFeature(t, 1, x);
      t->countFeature(t, 2, y);
    }
  else
    {
      x = t->GetFeature2(1);
      y = t->GetFeature2(2);
    }
  mark[2] = mark[2].setNum(x);
  mark[3] = mark[3].setNum(y+menuy);
  mark[4] = mark[4].setNum(t->GetFeature2(3));
  mark[5] = mark[5].setNum(t->GetFeature2(4));
  mark[6] = t->GetFeature1(4)->copy();
  mark[7] = "Font.PLAIN";
  if (t->GetFeature2(12) > 50) 
    mark[7].append("+Font.BOLD");
  if (t->GetFeature2(13) == 1) 
    mark[7].append("+Font.ITALIC");
  mark[8] = mark[8].setNum(t->GetFeature2(11));
  mark[9] = mark[9].setNum(t->GetFeature2(5));
  mark[10] = mark[10].setNum(t->GetFeature2(6));
  mark[11] = mark[11].setNum(t->GetFeature2(7));
  mark[12] = mark[12].setNum(t->GetFeature2(8));
  mark[13] = mark[13].setNum(t->GetFeature2(9));
  mark[14] = mark[14].setNum(t->GetFeature2(10));
  template2Source(homeDir+".build/tmp/choice.tmp", sysDir+"templates/choice.jbt", 15, mark, FALSE);
  
  QString buf;
  int sc = t->GetFeature2(18);
  if (sc == -1) sc = 0;
  buf = "        " + scope[sc] + " Choice " + temp1 + ";\n";
  insertBuffer(getTmpName(), getTargetName(), &buf, l1);
  l1++;
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  insertFile(getTmpName(), getTargetName(), homeDir+".build/tmp/choice.tmp", l2);
  copyFile(getTargetName(), getTmpName());
  l2 += 7;
  l3 += 7;
  int i, numch;
  numch = t->GetFeature2(20);
  QString buf2;
  for (i = 0; i < numch; i++)
    {
      progress->setProgress(counter++);
      
      buf2 = "        " + temp1.copy() + ".add( \"" + t->GetChildren(i)->GetFeature1(3)->copy() + "\" );\n";
      insertBuffer(getTmpName(), getTargetName(), &buf2, l2); 
      l2++;
      l3++;
      copyFile(getTargetName(), getTmpName());
    }
  buf2 = "        " + temp1.copy() + ".select( \"" + t->GetChildren(t->GetFeature2(21))->GetFeature1(3)->copy() + "\" );\n";
  insertBuffer(getTmpName(), getTargetName(), &buf2, l2);
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  buf2 = "        add( " + temp1.copy() + " );\n";
  insertBuffer(getTmpName(), getTargetName(), &buf2, l2);
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  if (t->GetFeature2(19) > 0)
      addListener(t, getTmpName(), getTargetName(), mark[0].copy(), l2);
}

void Generator::genList(MyTree *t)
{
  QString mark[16];

  QString temp1=t->GetFeature1(1)->copy();
  temp1 = temp1.right(temp1.length()-temp1.findRev(" ")-1);
  mark[0] = temp1.copy();
  mark[1] = "2";
  if (t->GetFeature2(21) == 1)
      mark[2] = "true";
  else mark[2] = "false";
  int x,y;
  x = 0;
  y = 0;
  if (*t->GetParent()->GetFeature1(2) == QString("MyCanvas"))
    {
      t->countFeature(t, 1, x);
      t->countFeature(t, 2, y);
    }
  else
    {
      x = t->GetFeature2(1);
      y = t->GetFeature2(2);
    }
  mark[3] = mark[3].setNum(x);
  mark[4] = mark[4].setNum(y+menuy);
  mark[5] = mark[5].setNum(t->GetFeature2(3));
  mark[6] = mark[6].setNum(t->GetFeature2(4));
  mark[7] = t->GetFeature1(4)->copy();
  mark[8] = "Font.PLAIN";
  if (t->GetFeature2(12) > 50) 
    mark[8].append("+Font.BOLD");
  if (t->GetFeature2(13) == 1) 
    mark[8].append("+Font.ITALIC");
  mark[9] = mark[9].setNum(t->GetFeature2(11));
  mark[10] = mark[10].setNum(t->GetFeature2(5));
  mark[11] = mark[11].setNum(t->GetFeature2(6));
  mark[12] = mark[12].setNum(t->GetFeature2(7));
  mark[13] = mark[13].setNum(t->GetFeature2(8));
  mark[14] = mark[14].setNum(t->GetFeature2(9));
  mark[15] = mark[15].setNum(t->GetFeature2(10));
  template2Source(homeDir+".build/tmp/list.tmp", sysDir+"templates/list.jbt", 16, mark, FALSE);
  
  QString buf;
  int sc = t->GetFeature2(18);
  if (sc == -1) sc = 0;
  buf = "        " + scope[sc] + " List " + temp1 + ";\n";
  insertBuffer(getTmpName(), getTargetName(), &buf, l1);
  l1++;
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  insertFile(getTmpName(), getTargetName(), homeDir+".build/tmp/list.tmp", l2);
  copyFile(getTargetName(), getTmpName());
  l2 += 7;
  l3 += 7;
  int i, numch;
  numch = t->GetFeature2(20);
  QString buf2;
  
  for (i = 0; i < numch; i++)
    {
      progress->setProgress(counter++);

      buf2 = "        " + temp1.copy() + ".add( \"" + t->GetChildren(i)->GetFeature1(3)->copy() + "\" );\n";
      bool select = FALSE;
      if (t->GetChildren(i)->GetFeature2(20) == 1)
	{
	  QString num;
	  num = num.setNum(i);
	  buf = buf + temp1.copy() + "        .select( " + num + " );\n";
	  select = TRUE;
	}
      insertBuffer(getTmpName(), getTargetName(), &buf2, l2); 
      l2++;
      l3++;
      if (select) 
	{
	  l2++;
	  l3++;
	}
      copyFile(getTargetName(), getTmpName());
    }
  buf2 = "        add( " + temp1.copy() + " );\n";
  insertBuffer(getTmpName(), getTargetName(), &buf2, l2);
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  if (t->GetFeature2(19) > 0)
      addListener(t, getTmpName(), getTargetName(), mark[0].copy(), l2);
}


void Generator::genPushButton(MyTree *t)
{
  QString mark[15];

  QString temp1=t->GetFeature1(1)->copy();
  temp1 = temp1.right(temp1.length()-temp1.findRev(" ")-1);
  mark[0] = temp1.copy();
  mark[1] = t->GetFeature1(3)->copy();
  int x,y;
  x = 0;
  y = 0;
  if (*t->GetParent()->GetFeature1(2) == QString("MyCanvas"))
    {
      t->countFeature(t, 1, x);
      t->countFeature(t, 2, y);
    }
  else
    {
      x = t->GetFeature2(1);
      y = t->GetFeature2(2);
    }
  mark[2] = mark[2].setNum(x);
  mark[3] = mark[3].setNum(y+menuy);
  mark[4] = mark[4].setNum(t->GetFeature2(3));
  mark[5] = mark[5].setNum(t->GetFeature2(4));
  mark[6] = t->GetFeature1(4)->copy();
  mark[7] = "Font.PLAIN";
  if (t->GetFeature2(12) > 50) 
    mark[7].append("+Font.BOLD");
  if (t->GetFeature2(13) == 1) 
    mark[7].append("+Font.ITALIC");
  mark[8] = mark[8].setNum(t->GetFeature2(11));
  mark[9] = mark[9].setNum(t->GetFeature2(5));
  mark[10] = mark[10].setNum(t->GetFeature2(6));
  mark[11] = mark[11].setNum(t->GetFeature2(7));
  mark[12] = mark[12].setNum(t->GetFeature2(8));
  mark[13] = mark[13].setNum(t->GetFeature2(9));
  mark[14] = mark[14].setNum(t->GetFeature2(10));
  template2Source(homeDir+".build/tmp/button.tmp", sysDir+"templates/button.jbt", 15, mark, FALSE);
  
  QString buf;
  int sc = t->GetFeature2(18);
  if (sc == -1) sc = 0;
  buf = "        "+ scope[sc] + " Button " + temp1 + ";\n";
  insertBuffer(getTmpName(), getTargetName(), &buf, l1);
  l1++;
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  insertFile(getTmpName(), getTargetName(), homeDir+".build/tmp/button.tmp", l2);
  copyFile(getTargetName(), getTmpName());
  l2 += 8;
  l3 += 8;
  if (t->GetFeature2(19) > 0)
      addListener(t, getTmpName(), getTargetName(), mark[0].copy(), l2);
}

void Generator::genScrollbar(MyTree *t)
{
  QString mark[16];

  QString temp1=t->GetFeature1(1)->copy();
  temp1 = temp1.right(temp1.length()-temp1.findRev(" ")-1);
  mark[0] = temp1.copy();
  if (t->GetFeature2(20) == 1)
    mark[1] = "Scrollbar.HORIZONTAL";
  else 
    mark[1] = "Scrollbar.VERTICAL";
  mark[2] = mark[2].setNum(t->GetFeature2(21));
  mark[3] = mark[3].setNum(t->GetFeature2(24));
  mark[4] = mark[4].setNum(t->GetFeature2(23));
  mark[5] = mark[5].setNum(t->GetFeature2(22));
  int x,y;
  x = 0;
  y = 0;
  if (*t->GetParent()->GetFeature1(2) == QString("MyCanvas"))
    {
      t->countFeature(t, 1, x);
      t->countFeature(t, 2, y);
    }
  else
    {
      x = t->GetFeature2(1);
      y = t->GetFeature2(2);
    }
  mark[6] = mark[6].setNum(x);
  mark[7] = mark[7].setNum(y+menuy);
  mark[8] = mark[8].setNum(t->GetFeature2(3));
  mark[9] = mark[9].setNum(t->GetFeature2(4));
  mark[10] = mark[10].setNum(t->GetFeature2(5));
  mark[11] = mark[11].setNum(t->GetFeature2(6));
  mark[12] = mark[12].setNum(t->GetFeature2(7));
  mark[13] = mark[13].setNum(t->GetFeature2(8));
  mark[14] = mark[14].setNum(t->GetFeature2(9));
  mark[15] = mark[15].setNum(t->GetFeature2(10));
  template2Source(homeDir+".build/tmp/scrollbar.tmp", sysDir+"templates/scrollbar.jbt", 16, mark, FALSE);
  
  QString buf;
  int sc = t->GetFeature2(18);
  if (sc == -1) sc = 0;
  buf = "        " + scope[sc] + " Scrollbar " + temp1 + ";\n";
  insertBuffer(getTmpName(), getTargetName(), &buf, l1);
  l1++;
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  insertFile(getTmpName(), getTargetName(), homeDir+".build/tmp/scrollbar.tmp", l2);
  copyFile(getTargetName(), getTmpName());
  l2 += 7;
  l3 += 7;
  if (t->GetFeature2(19) > 0)
      addListener(t, getTmpName(), getTargetName(), mark[0].copy(), l2);
}

void Generator::genMenuItem(MyTree *t, QString higher)
{
  QString mark[3];

  QString n1(t->GetFeature1(3)->data()), n2;
  n2 = n1.stripWhiteSpace();
  n1 = n2.replace(QRegExp(" "), "");
  mark[0] = n1 + "_";
  mark[1] = t->GetFeature1(3)->stripWhiteSpace();
  QString buf;
  int sc = t->GetFeature2(18);
  if (sc == -1) sc = 0;
  if (!mark[0].isEmpty()) 
    buf = "        " + scope[sc] + " MenuItem " + mark[0].copy() + ";\n";
  mark[2] = higher.copy();
  template2Source(homeDir+".build/tmp/menuitem.tmp", sysDir+"templates/menuitem.jbt", 3, mark, FALSE);
  
  insertBuffer(getTmpName(), getTargetName(), &buf, l1);
  l1++;
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  insertFile(getTmpName(), getTargetName(), homeDir+".build/tmp/menuitem.tmp", l2);
  l2 += 3;
  l3 += 3;
  if (!t->GetFeature1(6)->isEmpty())
    {
      copyFile(getTargetName(), getTmpName());
      QString key;
      key = key.insert(0, (char) (t->GetFeature2(20)));
      buf = "        " + mark[0].copy() + ".setShortcut( new MenuShortcut( KeyEvent.VK_" + key.copy() + " ) );\n";
      insertBuffer(getTmpName(), getTargetName(), &buf, l2);
      l2++;
      l3++;
    }
}

void Generator::genSubMenu(MyTree *t, QString higher)
{
  QString mark[3];

  QString n1(t->GetFeature1(3)->data()), n2;
  n2 = n1.stripWhiteSpace();
  n1 = n2.replace(QRegExp(" "), "");
  mark[0] = n1 + "_";
  mark[1] = higher.copy();
  mark[2] = t->GetFeature1(3)->stripWhiteSpace();
  QString buf;
  int sc = t->GetFeature2(18);
  if (sc == -1) sc = 0;
  if (!mark[0].isEmpty()) 
    buf = "        "+ scope[sc] + " Menu " + mark[0].copy() + ";\n";
  template2Source(homeDir+".build/tmp/submenu.tmp", sysDir+"templates/submenu.jbt", 3, mark, FALSE);
  
  insertBuffer(getTmpName(), getTargetName(), &buf, l1);
  l1++;
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  insertFile(getTmpName(), getTargetName(), homeDir+".build/tmp/submenu.tmp", l2);
  l2 += 3;
  l3 += 3;
  if (!t->GetFeature1(6)->isEmpty())
    {
      copyFile(getTargetName(), getTmpName());
      QString key;
      key = key.insert(0, (char) (t->GetFeature2(20)));
      buf = "        " + mark[0].copy() + ".setShortcut( new MenuShortcut( KeyEvent.VK_" + key.copy() + " ) );\n";
      insertBuffer(getTmpName(), getTargetName(), &buf, l2);
      l2++;
      l3++;
    }
}

void Generator::genCheckboxMenuItem(MyTree *t, QString higher)
{
  QString mark[4];

  QString n1(t->GetFeature1(3)->data()), n2;
  n2 = n1.stripWhiteSpace();
  n1 = n2.replace(QRegExp(" "), "");
  mark[0] = n1 + "_";
  mark[1] = t->GetFeature1(3)->stripWhiteSpace();
  if (t->GetFeature2(23)) 
    mark[3] = "true";
  else
    mark[3] = "false";
  QString buf;
  int sc = t->GetFeature2(18);
  if (sc == -1) sc = 0;
  if (!mark[0].isEmpty()) 
    buf = "        " + scope[sc] + " CheckboxMenuItem " + mark[0].copy() + ";\n";
  mark[2] = higher.copy();
  template2Source(homeDir+".build/tmp/chmenuitem.tmp", sysDir+"templates/chmenuitem.jbt", 4, mark, FALSE);
  
  insertBuffer(getTmpName(), getTargetName(), &buf, l1);
  l1++;
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  insertFile(getTmpName(), getTargetName(), homeDir+".build/tmp/chmenuitem.tmp", l2);
  l2 += 3;
  l3 += 3;
  if (!t->GetFeature1(6)->isEmpty())
    {
      copyFile(getTargetName(), getTmpName());
      QString key;
      key = key.insert(0, (char) (t->GetFeature2(20)));
      buf = "        " + mark[0].copy() + ".setShortcut( new MenuShortcut( KeyEvent.VK_" + key.copy() + " ) );\n";
      insertBuffer(getTmpName(), getTargetName(), &buf, l2);
      l2++;
      l3++;
    }
}

void Generator::genMenu(MyTree *t)
{
  QString mark[6];

  QString temp1=t->GetFeature1(1)->copy();
  temp1 = temp1.right(temp1.length()-temp1.findRev(" ")-1);
  mark[0] = temp1.copy();
  mark[1] = t->GetFeature1(3)->copy();
  QString buf;
  int sc = t->GetFeature2(18);
  if (sc == -1) sc = 0;
  buf = "        " + scope[sc] + " Menu " + temp1 + ";\n";
  temp1 = t->GetParent()->GetFeature1(1)->copy();
  temp1 = temp1.right(temp1.length()-temp1.findRev(" ")-1);
  mark[2] = temp1.copy();
  mark[3] = t->GetFeature1(4)->copy();
  mark[4] = "Font.PLAIN";
  if (t->GetFeature2(12) > 50) 
    mark[4].append("+Font.BOLD");
  if (t->GetFeature2(13) == 1) 
    mark[4].append("+Font.ITALIC");
  mark[5] = mark[5].setNum(t->GetFeature2(11)); 
  template2Source(homeDir+".build/tmp/menu.tmp", sysDir+"templates/menu.jbt", 6, mark, FALSE);
  
  insertBuffer(getTmpName(), getTargetName(), &buf, l1);
  l1++;
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  insertFile(getTmpName(), getTargetName(), homeDir+".build/tmp/menu.tmp", l2);
  l2 += 4;
  l3 += 4;
  int i;
  int numch = t->GetFeature2(22);
  for (i=0; i<numch; i++)
    {
      progress->setProgress(counter++);

      QString higher = getHigherMenu(t, t->GetChildren(i)->GetFeature1(3)->data(), i);
      if (t->GetChildren(i)->GetFeature2(24) == 2) // separator
	{
	  copyFile(getTargetName(), getTmpName());
	  buf = "        " + mark[0].copy() + ".addSeparator();\n";
	  insertBuffer(getTmpName(), getTargetName(), &buf, l2);
	  l2++;
	  l3++;
	}
      else
	{
	  if (t->GetChildren(i)->GetFeature2(24) != 1) // not a submenu
	    {
	      if (t->GetChildren(i)->GetFeature2(22)) // checkboxmenuitem
		{
		  copyFile(getTargetName(), getTmpName());
		  genCheckboxMenuItem(t->GetChildren(i), higher);
		}
	      else 
		if (!t->GetChildren(i)->GetFeature2(24)) // menuitem
		  {
		    copyFile(getTargetName(), getTmpName());
		    genMenuItem(t->GetChildren(i), higher);
		  }
	    }
	  else // submenu
	    {
	      copyFile(getTargetName(), getTmpName());
	      genSubMenu(t->GetChildren(i), higher);
	    }
	}
    }
  
  if (t->GetFeature2(23) == 1)
    {
      copyFile(getTargetName(), getTmpName());
      buf = "        " + mark[2].copy() + ".setHelpMenu( " + mark[0].copy() + " );\n";
      insertBuffer(getTmpName(), getTargetName(), &buf, l2);
      l2++;
      l3++;
    }
  if (!t->GetFeature1(6)->isEmpty())
    {
      copyFile(getTargetName(), getTmpName());
      QString key;
      key = key.insert(0, (char) (t->GetFeature2(20)));
      buf = "        " + mark[0].copy() + ".setShortcut( new MenuShortcut( KeyEvent.VK_" + key.copy() + " ) );\n";
      insertBuffer(getTmpName(), getTargetName(), &buf, l2);
      l2++;
      l3++;
    }
  copyFile(getTargetName(), getTmpName());
  if (t->GetFeature2(19) > 0)
      addListener(t, getTmpName(), getTargetName(), mark[0].copy(), l2);
}
  
void Generator::genMenuBar(MyTree *t)
{
  QString mark[1];

  QString temp1=t->GetFeature1(1)->copy();
  temp1 = temp1.right(temp1.length()-temp1.findRev(" ")-1);
  mark[0] = temp1.copy();
  template2Source(homeDir+".build/tmp/menubar.tmp", sysDir+"templates/menubar.jbt", 1, mark, FALSE);
  
  QString buf;
  int sc = t->GetFeature2(18);
  if (sc == -1) sc = 0;
  buf = "        " + scope[sc] + " MenuBar " + temp1 + ";\n";
  insertBuffer(getTmpName(), getTargetName(), &buf, l1);
  l1++;
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  insertFile(getTmpName(), getTargetName(), homeDir+".build/tmp/menubar.tmp", l2);
  copyFile(getTargetName(), getTmpName());
  l2 += 2;
  l3 += 3;
  if (t->GetFeature2(19) > 0)
      addListener(t, getTmpName(), getTargetName(), mark[0].copy(), l2);
}


void Generator::genLabel(MyTree *t)
{
  QString mark[16];

  QString temp1=t->GetFeature1(1)->copy();
  temp1 = temp1.right(temp1.length()-temp1.findRev(" ")-1);
  mark[0] = temp1.copy();
  mark[1] = t->GetFeature1(3)->copy();
  if (t->GetFeature2(20) == 0)
    mark[2] = "Label.LEFT";
  else 
    if (t->GetFeature2(20) == 1) mark[2] = "Label.CENTER";
    else mark[2] = "Label.RIGHT";
  int x,y;
  x = 0;
  y = 0;
  if (*t->GetParent()->GetFeature1(2) == QString("MyCanvas"))
    {
      t->countFeature(t, 1, x);
      t->countFeature(t, 2, y);
    }
  else
    {
      x = t->GetFeature2(1);
      y = t->GetFeature2(2);
    }
  mark[3] = mark[3].setNum(x);
  mark[4] = mark[4].setNum(y+menuy);
  mark[5] = mark[5].setNum(t->GetFeature2(3));
  mark[6] = mark[6].setNum(t->GetFeature2(4));
  mark[7] = mark[7].setNum(t->GetFeature2(5));
  mark[8] = mark[8].setNum(t->GetFeature2(6));
  mark[9] = mark[9].setNum(t->GetFeature2(7));
  mark[10] = mark[10].setNum(t->GetFeature2(8));
  mark[11] = mark[11].setNum(t->GetFeature2(9));
  mark[12] = mark[12].setNum(t->GetFeature2(10));
  mark[13] = t->GetFeature1(4)->copy();
  mark[14] = "Font.PLAIN";
  if (t->GetFeature2(12) > 50) 
    mark[14].append("+Font.BOLD");
  if (t->GetFeature2(13) == 1) 
    mark[14].append("+Font.ITALIC");
  mark[15] = mark[15].setNum(t->GetFeature2(11)); 
  template2Source(homeDir+".build/tmp/label.tmp", sysDir+"templates/label.jbt", 16, mark, FALSE);
  
  QString buf;
  int sc = t->GetFeature2(18);
  if (sc == -1) sc = 0;
  buf = "        " + scope[sc] + " Label " + temp1 + ";\n";
  insertBuffer(getTmpName(), getTargetName(), &buf, l1);
  l1++;
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  insertFile(getTmpName(), getTargetName(), homeDir+".build/tmp/label.tmp", l2);
  copyFile(getTargetName(), getTmpName());
  l2 += 8;
  l3 += 8;
  if (t->GetFeature2(19) > 0)
      addListener(t, getTmpName(), getTargetName(), mark[0].copy(), l2);
}

void Generator::genTextField(MyTree *t)
{
  QString mark[17];

  QString temp1=t->GetFeature1(1)->copy();
  temp1 = temp1.right(temp1.length()-temp1.findRev(" ")-1);
  mark[0] = temp1.copy();
  mark[1] = t->GetFeature1(3)->copy();
  if (t->GetFeature1(5)->isEmpty() == FALSE)
    mark[16] = t->GetFeature1(5)->copy();
  else 
    mark[16] = "\\0";
  if (t->GetFeature2(20) == 1)
    mark[2] = "true";
  else
    mark[2] = "false";
  int x,y;
  x = 0;
  y = 0;
  if (*t->GetParent()->GetFeature1(2) == QString("MyCanvas"))
    {
      t->countFeature(t, 1, x);
      t->countFeature(t, 2, y);
    }
  else
    {
      x = t->GetFeature2(1);
      y = t->GetFeature2(2);
    }
  mark[3] = mark[3].setNum(x);
  mark[4] = mark[4].setNum(y+menuy);
  mark[5] = mark[5].setNum(t->GetFeature2(3));
  mark[6] = mark[6].setNum(t->GetFeature2(4)+(t->GetFeature2(11)-5));
  mark[7] = mark[7].setNum(t->GetFeature2(5));
  mark[8] = mark[8].setNum(t->GetFeature2(6));
  mark[9] = mark[9].setNum(t->GetFeature2(7));
  mark[10] = mark[10].setNum(t->GetFeature2(8));
  mark[11] = mark[11].setNum(t->GetFeature2(9));
  mark[12] = mark[12].setNum(t->GetFeature2(10));
  mark[13] = t->GetFeature1(4)->copy();
  mark[14] = "Font.PLAIN";
  if (t->GetFeature2(12) > 50) 
    mark[14].append("+Font.BOLD");
  if (t->GetFeature2(13) == 1) 
    mark[14].append("+Font.ITALIC");
  mark[15] = mark[15].setNum(t->GetFeature2(11)); 
  template2Source(homeDir+".build/tmp/textfield.tmp", sysDir+"templates/textfield.jbt", 17, mark, FALSE);
  
  QString buf;
  int sc = t->GetFeature2(18);
  if (sc == -1) sc = 0;
  buf = "        " + scope[sc] + " TextField " + temp1 + ";\n";
  insertBuffer(getTmpName(), getTargetName(), &buf, l1);
  l1++;
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  insertFile(getTmpName(), getTargetName(), homeDir+".build/tmp/textfield.tmp", l2);
  copyFile(getTargetName(), getTmpName());
  l2 += 10;
  l3 += 10;
  if (t->GetFeature2(19) > 0)
      addListener(t, getTmpName(), getTargetName(), mark[0].copy(), l2);
}

void Generator::genTextArea(MyTree *t)
{
  QString mark[17];

  QString temp1=t->GetFeature1(1)->copy();
  temp1 = temp1.right(temp1.length()-temp1.findRev(" ")-1);
  mark[0] = temp1.copy();
  QString name = homeDir+".build/tmp/" + temp1.copy() + ".tmp";
  QFile f2(name);
  if (f2.open(IO_ReadOnly) == FALSE)
    {
      mark[1] = "";
    }
  else
    {
      QTextStream t( &f2 );
      QString s;          
      int numline = 0;
      while ( !t.eof() ) 
	{
	  s = t.readLine(); 
	  if (numline++) mark[1].append("\\n");
	  mark[1].append((const char*) s);
	}  
      f2.close();
    }
  switch (t->GetFeature2(21)) 
    {
    case 0:
      mark[16] = "TextArea.SCROLLBARS_BOTH";
      break;
    case 1:
      mark[16] = "TextArea.SCROLLBARS_HORIZONTAL_ONLY";
      break;
    case 2:
      mark[16] = "TextArea.SCROLLBARS_NONE";
      break;
    case 3:
      mark[16] = "TextArea.SCROLLBARS_VERTICAL_ONLY";
    default:
      mark[16] = "TextArea.SCROLLBARS_NONE";
    }
  if (t->GetFeature2(20) == 1)
    mark[2] = "true";
  else
    mark[2] = "false";
  int x,y;
  x = 0;
  y = 0;
  if (*t->GetParent()->GetFeature1(2) == QString("MyCanvas"))
    {
      t->countFeature(t, 1, x);
      t->countFeature(t, 2, y);
    }
  else
    {
      x = t->GetFeature2(1);
      y = t->GetFeature2(2);
    }
  mark[3] = mark[3].setNum(x);
  mark[4] = mark[4].setNum(y+menuy);
  mark[5] = mark[5].setNum(t->GetFeature2(3));
  mark[6] = mark[6].setNum(t->GetFeature2(4)+(t->GetFeature2(11)-5));
  mark[7] = mark[7].setNum(t->GetFeature2(5));
  mark[8] = mark[8].setNum(t->GetFeature2(6));
  mark[9] = mark[9].setNum(t->GetFeature2(7));
  mark[10] = mark[10].setNum(t->GetFeature2(8));
  mark[11] = mark[11].setNum(t->GetFeature2(9));
  mark[12] = mark[12].setNum(t->GetFeature2(10));
  mark[13] = t->GetFeature1(4)->copy();
  mark[14] = "Font.PLAIN";
  if (t->GetFeature2(12) > 50) 
    mark[14].append("+Font.BOLD");
  if (t->GetFeature2(13) == 1) 
    mark[14].append("+Font.ITALIC");
  mark[15] = mark[15].setNum(t->GetFeature2(11)); 
  template2Source(homeDir+".build/tmp/textarea.tmp", sysDir+"templates/textarea.jbt", 17, mark, FALSE);
  
  QString buf;
  int sc = t->GetFeature2(18);
  if (sc == -1) sc = 0;
  buf = "        " + scope[sc] + " TextArea " + temp1 + ";\n";
  insertBuffer(getTmpName(), getTargetName(), &buf, l1);
  l1++;
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  insertFile(getTmpName(), getTargetName(), homeDir+".build/tmp/textarea.tmp", l2);
  copyFile(getTargetName(), getTmpName());
  l2 += 9;
  l3 += 9;
  if (t->GetFeature2(19) > 0)
      addListener(t, getTmpName(), getTargetName(), mark[0].copy(), l2);
}


void Generator::addNewCanvas(QString name)
{
  if (numcanvas < 20)
    canvastab[numcanvas++] = name.copy();
}

void Generator::genCanvas(MyTree *t)
{
  QString mark[11];

  QString temp1=t->GetFeature1(1)->copy();
  temp1 = temp1.right(temp1.length()-temp1.findRev(" ")-1);
  addNewCanvas(temp1.copy());
  mark[0] = temp1.copy();
  int x,y;
  x = 0;
  y = 0;
  if (*t->GetParent()->GetFeature1(2) == QString("MyCanvas"))
    {
      t->countFeature(t, 1, x);
      t->countFeature(t, 2, y);
    }
  else
    {
      x = t->GetFeature2(1);
      y = t->GetFeature2(2);
    }
  mark[1] = mark[1].setNum(x);
  mark[2] = mark[2].setNum(y+menuy);
  mark[3] = mark[3].setNum(t->GetFeature2(3));
  mark[4] = mark[4].setNum(t->GetFeature2(4));
  mark[5] = mark[5].setNum(t->GetFeature2(5));
  mark[6] = mark[6].setNum(t->GetFeature2(6));
  mark[7] = mark[7].setNum(t->GetFeature2(7));
  mark[8] = mark[8].setNum(t->GetFeature2(8));
  mark[9] = mark[9].setNum(t->GetFeature2(9));
  mark[10] = mark[10].setNum(t->GetFeature2(10));
  template2Source(homeDir+".build/tmp/canvas.tmp", sysDir+"templates/canvas.jbt", 11, mark, FALSE);
  
  QString buf;
  int sc = t->GetFeature2(18);
  if (sc == -1) sc = 0;
  buf = "        " + scope[sc] + " Canvas " + temp1 + ";\n";
  insertBuffer(getTmpName(), getTargetName(), &buf, l1);
  l1++;
  l2++;
  l3++;
  copyFile(getTargetName(), getTmpName());
  insertFile(getTmpName(), getTargetName(), homeDir+".build/tmp/canvas.tmp", l2);
  copyFile(getTargetName(), getTmpName());
  l2 += 6;
  l3 += 6;
  if (t->GetFeature2(19) > 0)
      addListener(t, getTmpName(), getTargetName(), mark[0].copy(), l2);
}

int Generator::generateSource(MyTree *t)
{
  //only for progress
  QString tmp, var_name;
  progress->setProgress(counter++);
  tmp=QString(t->GetFeature1(1)->data());
  var_name=tmp.right(tmp.length()-tmp.find(" ")-1);
  QString whatname;
  int n = 0;
  if (*t->GetFeature1(2) == "MyClass") {
    menuy = 0;
    QString *menubar[1];
    menubar[0] = new QString(3);
    t->FindFeature(t, 2, "MyMenuBar", 0, menubar, n);
    switch(t->GetFeature2(20))
      {
      case AP:
	if (n)
	  if (QMessageBox::warning(0, "Generating Source Warning", "You have MenuBar defined in your project, which extends Applet.\nDo you want to continue?", "Yes", "No", 0, 1)) return -1;
	setTargetName(homeDir+".build/tmp/applet.tmp");
	menuy = 30;
	genApplet(t);
	if (n) menuy += 8; 
	whatname = "applet.tmp";
	break;
      case FR:
	menuy = 30;
	setTargetName(homeDir+".build/tmp/frame.tmp");
	genFrame(t);
	if (n) menuy += 8; 
	whatname = "frame.tmp";
	break;
      case WN:
	setTargetName(homeDir+".build/tmp/window.tmp");
	menuy = 30;
	genWindow(t);
	whatname = "window.tmp";
      }
  }
  if (*t->GetFeature1(2) == "MyPushButton") {
    genPushButton(t);
    return 1;
  }
  if (*t->GetFeature1(2) == "MyRadioButton") {
    genCheckbox(t);
    return 1;
  }
  if (*t->GetFeature1(2) == "MyScrollBar") {
    genScrollbar(t);
    return 1;
  }
  if (*t->GetFeature1(2) == "MyLabel") {
    genLabel(t);
    return 1;
  }
  if (*t->GetFeature1(2) == "MyChoice") {
    genChoice(t);
    return 1;
  }
  if (*t->GetFeature1(2) == "MyList") {
    genList(t);
    return 1;
  }
  if (*t->GetFeature1(2) == "MyTextField") {
    genTextField(t);
    return 1;
  }
  if (*t->GetFeature1(2) == "MyTextArea") {
    genTextArea(t);
    return 1;
  }
  if (*t->GetFeature1(2) == "MyMenuBar") {
    genMenuBar(t);
  }
  if (*t->GetFeature1(2) == "MyMenu") {
    genMenu(t);
    return 1;
  }
  if (*t->GetFeature1(2) == "MyCanvas") 
    genCanvas(t);
  int j, numch;
  numch = t->GetNumChildren();
  if (numch)
    for (j = 0; j < numch; j++) 
      generateSource(t->GetChildren(j));
  if (*t->GetFeature1(2) == "MyClass")
    {
      if (numcanvas)
	{
	  int i;
	  QString buf;
	  buf = "";
	  for (i = numcanvas - 1; i >= 0; i--)
	    buf = buf + "        add(" + canvastab[i].copy() + ");\n";
	  insertBuffer(getTmpName(), getTargetName(), &buf, l2);
	  l2 += numcanvas;
	  l3 += numcanvas;
	  copyFile(getTargetName(), getTmpName());
	}
      if (!numch)
	{
	  removeSpace(homeDir+".build/tmp/last.tmp", homeDir+".build/tmp/"+whatname);
	  genComment(homeDir, var_name);
	  copyFile(homeDir+".build/out/"+var_name+".java", homeDir+".build/tmp/comment.tmp");
	  return 0;
	}
    }
  return 1;
}

void Generator::genComment(QString homename, QString name)
{
  QFile f(homename+".build/tmp/comment.tmp");
  QTime time(QTime::currentTime());
  QDate date(QDate::currentDate());
  QString mark[4];
  QString temp2 = name.copy();
  QString temp1 = temp2.right(temp2.length()-temp2.findRev("/") - 1);
  mark[0] = temp1.copy();
  mark[1] = "Java 1.1";
  mark[2] = time.toString() + "   " + date.toString();
  mark[3] = VERSION;
  template2Source(homeDir+".build/tmp/comment.tmp", sysDir+"templates/comment.jbt", 4, mark, FALSE);
  concateFiles(homeDir+".build/tmp/comment.tmp", homename+".build/tmp/last.tmp", TRUE);
}

void Generator::finishSource(QString homename, QString whatname, QString name)
{
  copyFile(homename+".build/tmp/last.tmp",whatname);
  if (style == 0) addImplements(homename+".build/tmp/last.tmp"); // for old style
  removeSpace(name, homename+".build/tmp/last.tmp");
  if (style == 0) addImport(name);  // for old style
  removeSpace(homename+".build/tmp/last.tmp", name);
  genComment(homename, name);
  copyFile(name, homename+".build/tmp/comment.tmp");
}

void Generator::genHtml(MyTree *t)
{
  QString mark[3];
  QString temp1=t->GetFeature1(1)->copy();
  temp1 = temp1.right(temp1.length()-temp1.findRev(" ")-1);
  mark[0] = temp1.copy();
  mark[1] = mark[1].setNum(t->GetFeature2(3));
  mark[2] = mark[2].setNum(t->GetFeature2(4));
  template2Source(homeDir+".build/out/" + temp1.copy()+".html", sysDir+"templates/html.jbt", 17, mark, FALSE);  
}
